from uuid import uuid4

from flask import send_file
from werkzeug.exceptions import NotFound

from api.route import Route
from core.logger import APILogger

logger = APILogger(__name__)


@Route.get(f"/api/files/<path:file_path>")
def get_file(file_path: str):
    name = file_path
    if "/" in file_path:
        name = file_path.split("/")[-1]

    try:
        return send_file(
            f"../files/{file_path}", download_name=name, as_attachment=True
        )
    except NotFound:
        return {"error": "File not found"}, 404
    except Exception as e:
        error_id = uuid4()
        logger.error(f"Error {error_id} getting file {file_path}", e)
        return {"error": f"File error. ErrorId: {error_id}"}, 500
