from flask import redirect, jsonify, request, Response

from api.route import Route
from core.logger import Logger
from data.schemas.public.short_url import ShortUrl
from data.schemas.public.short_url_dao import shortUrlDao
from data.schemas.public.short_url_visit import ShortUrlVisit
from data.schemas.public.short_url_visit_dao import shortUrlVisitDao

logger = Logger(__name__)


@Route.get(f"/api/find/<path:path>")
async def find(path: str):
    from_db = await shortUrlDao.find_single_by({ShortUrl.short_url: path})
    if from_db is None:
        return jsonify(None)

    return jsonify(from_db.to_dto())


@Route.get(f"/api/redirect/<path:path>")
async def handle_short_url(path: str):
    if path.startswith("api/"):
        path = path.replace("api/", "")

    from_db = await shortUrlDao.find_single_by({ShortUrl.short_url: path})
    if from_db is None:
        return {"error": "Short URL not found"}, 404

    try:
        await shortUrlVisitDao.create(
            ShortUrlVisit(
                0,
                from_db.id,
                request.headers.get("User-Agent")
            )
        )
    except Exception as e:
        logger.error(f"Failed to update short url {from_db.short_url} with error", e)

    return _do_redirect(from_db.target_url)

def _do_redirect(url: str) -> Response:
    # todo: multiple protocols like ts3://
    if not url.startswith("http://") and not url.startswith("https://"):
        url = f"http://{url}"

    return redirect(url)
