from api_graphql.abc.mutation_abc import MutationABC
from api_graphql.input.group_create_input import GroupCreateInput
from api_graphql.input.group_update_input import GroupUpdateInput
from core.logger import APILogger
from data.schemas.public.group import Group
from data.schemas.public.group_dao import groupDao
from service.permission.permissions_enum import Permissions

logger = APILogger(__name__)


class GroupMutation(MutationABC):
    def __init__(self):
        MutationABC.__init__(self, "Group")

        self.mutation(
            "create",
            self.resolve_create,
            GroupCreateInput,
            require_any_permission=[Permissions.groups_create],
        )
        self.mutation(
            "update",
            self.resolve_update,
            GroupUpdateInput,
            require_any_permission=[Permissions.groups_update],
        )
        self.mutation(
            "delete",
            self.resolve_delete,
            require_any_permission=[Permissions.groups_delete],
        )
        self.mutation(
            "restore",
            self.resolve_restore,
            require_any_permission=[Permissions.groups_delete],
        )

    @staticmethod
    async def resolve_create(obj: GroupCreateInput, *_):
        logger.debug(f"create group: {obj.__dict__}")

        group = Group(
            0,
            obj.name,
        )
        nid = await groupDao.create(group)
        return await groupDao.get_by_id(nid)

    @staticmethod
    async def resolve_update(obj: GroupUpdateInput, *_):
        logger.debug(f"update group: {input}")

        if obj.name is not None:
            group = await groupDao.get_by_id(obj.id)
            group.name = obj.name
            await groupDao.update(group)

        return await groupDao.get_by_id(obj.id)

    @staticmethod
    async def resolve_delete(*_, id: str):
        logger.debug(f"delete group: {id}")
        group = await groupDao.get_by_id(id)
        await groupDao.delete(group)
        return True

    @staticmethod
    async def resolve_restore(*_, id: str):
        logger.debug(f"restore group: {id}")
        group = await groupDao.get_by_id(id)
        await groupDao.restore(group)
        return True
