import { Component, OnInit } from '@angular/core';
import { SidebarService } from 'src/app/service/sidebar.service';
import { Router } from '@angular/router';
import { HttpClient } from '@angular/common/http';
import { ShortUrlDto } from 'src/app/model/entities/short-url';
import { SettingsService } from 'src/app/service/settings.service';
import { AppSettings } from 'src/app/model/config/app-settings';
import { GuiService } from 'src/app/service/gui.service';

@Component({
  selector: 'app-redirect',
  templateUrl: './redirect.component.html',
  styleUrl: './redirect.component.scss',
})
export class RedirectComponent implements OnInit {
  defaultSecs = 5;
  secs = -1;

  settings: AppSettings = this.settingsService.settings;

  constructor(
    private sidebar: SidebarService,
    private router: Router,
    private http: HttpClient,
    private settingsService: SettingsService,
    private gui: GuiService
  ) {
    this.sidebar.hide();
    this.gui.hide();
  }

  ngOnInit() {
    this.handleUrl();
  }

  handleUrl() {
    let shortUrl = this.router.url;
    if (shortUrl === '/') {
      return;
    }

    if (shortUrl.startsWith('/')) {
      shortUrl = shortUrl.substring(1);
    }

    this.http
      .get<ShortUrlDto | undefined>(`${this.settings.api.url}/find/${shortUrl}`)
      .subscribe(async response => {
        if (!response) {
          await this.router.navigate(['/404']);
          return;
        }

        if (!response.loadingScreen) {
          window.location.href = `${this.settings.api.url}/redirect/${shortUrl}`;
          return;
        }

        this.secs = this.defaultSecs;
        setInterval(() => {
          this.secs--;

          if (this.secs === 0) {
            window.location.href = `${this.settings.api.url}/redirect/${shortUrl}`;
          }
        }, 1000);
      });
  }
}
